import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.concurrent.TimeUnit;
import java.util.ArrayList;
import java.util.List;

public class IOBenchmark {
    private static final int ITERATIONS = 5;
    private static final int LINES_TO_WRITE = 1_000_000;
    private static final String LINE = "OOP je super, mozno nie vzdy, moznooooo...\n";

    public static void main(String[] args) throws IOException {
        benchmarkFileWriter();
        benchmarkBufferedWriter();
    }

    private static void benchmarkFileWriter() throws IOException {
        List<Long> durations = new ArrayList<>();
        long totalTime = 0;

        for (int i = 0; i < ITERATIONS; i++) {
            Path tempFile = Files.createTempFile("filewriter_benchmark", ".txt");

            long startTime = System.nanoTime();

            try (FileWriter fw = new FileWriter(tempFile.toFile())) {
                for (int j = 0; j < LINES_TO_WRITE; j++) {
                    fw.write(LINE);
                }
            }

            long duration = System.nanoTime() - startTime;
            durations.add(duration);
            totalTime += duration;

            Files.deleteIfExists(tempFile);
        }

        printResults("FileWriter", durations, totalTime);
    }

    private static void benchmarkBufferedWriter() throws IOException {
        List<Long> durations = new ArrayList<>();
        long totalTime = 0;

        for (int i = 0; i < ITERATIONS; i++) {
            Path tempFile = Files.createTempFile("bufferedwriter_benchmark", ".txt");

            long startTime = System.nanoTime();

            try (BufferedWriter bw = new BufferedWriter(new FileWriter(tempFile.toFile()))) {
                for (int j = 0; j < LINES_TO_WRITE; j++) {
                    bw.write(LINE);
                }
            }

            long duration = System.nanoTime() - startTime;
            durations.add(duration);
            totalTime += duration;

            Files.deleteIfExists(tempFile);
        }

        printResults("BufferedWriter", durations, totalTime);
    }

    private static void printResults(String name, List<Long> durations, long totalTime) {
        long avgTime = totalTime / ITERATIONS;
        double stdev = calculateStandardDeviation(durations);

        System.out.println("\n" + name + " Benchmark Results:");
        System.out.println("--------------------------------");
        System.out.println("Iterations: " + ITERATIONS);
        System.out.println("Lines written per iteration: " + LINES_TO_WRITE);
        System.out.println("Total time: " + TimeUnit.NANOSECONDS.toMillis(totalTime) + " ms");
        System.out.println("Average time: " + TimeUnit.NANOSECONDS.toMillis(avgTime) + " ms");
        System.out.println("Standard deviation: " + TimeUnit.NANOSECONDS.toMillis((long)stdev) + " ms");
        System.out.println("Min time: " + TimeUnit.NANOSECONDS.toMillis(durations.stream().min(Long::compare).get()) + " ms");
        System.out.println("Max time: " + TimeUnit.NANOSECONDS.toMillis(durations.stream().max(Long::compare).get()) + " ms");
        System.out.println("--------------------------------");
    }

    private static double calculateStandardDeviation(List<Long> durations) {
        double mean = durations.stream().mapToLong(Long::longValue).average().orElse(0.0);
        double variance = durations.stream()
                .mapToDouble(d -> Math.pow(d - mean, 2))
                .average()
                .orElse(0.0);
        return Math.sqrt(variance);
    }
}